package com.gs.apidemo;

import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.ToneGenerator;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.gs.phone.api.audio.AudioRouteApi;
import com.gs.phone.api.call.BaseCallApi;
import com.gs.phone.api.setting.CallSettingApi;
import com.gs.phone.context.PhoneContext;
import com.gs.phone.entity.BaseLine;
import com.gs.phone.listener.CallStatusListener;

import java.util.List;

public class TalkingActivity extends BaseActivity implements View.OnClickListener {
    private static final String TAG = "TalkingActivity";
    private TextView mNumberTv, mNameTv, mStatusTv;
    private Button mAcceptBtn, mEndBtn;
    private int mPrevStatus = -1;
    private BaseLine mPrevLine;
    private MediaPlayer mMediaPlayer;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_talking);
        mNumberTv = (TextView) findViewById(R.id.tv_number);
        mNameTv = (TextView) findViewById(R.id.tv_name);
        mStatusTv = (TextView) findViewById(R.id.tv_line_status);
        mAcceptBtn = (Button) findViewById(R.id.btn_accept);
        mAcceptBtn.setOnClickListener(this);
        mEndBtn = (Button) findViewById(R.id.btn_end);
        mEndBtn.setOnClickListener(this);

        mPrevLine = null;
        mPrevStatus = -1;
        startMonitorCallLines();
        initializeTone();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        stopMonitorCallLines();
        releaseTone();
    }

    @Override
    public void onClick(View v) {
        if (mPrevLine != null) {
            if (R.id.btn_accept == v.getId() && mPrevLine.getStatus() == 2) {
                BaseCallApi.acceptCall(mPrevLine.getLineId(), false, false);
            } else if (R.id.btn_end == v.getId() && mPrevLine.getStatus() != 0) {
                BaseCallApi.endCall(mPrevLine.getLineId());
            }
        }
    }

    private ToneGenerator mToneGenerator;
    private void initializeTone() {
        try {
            mToneGenerator = new ToneGenerator(AudioManager.STREAM_SYSTEM, ToneGenerator.MAX_VOLUME * 8 /10);
        } catch (Throwable ex) {
            Log.w(TAG, "Exception caught while creating tone generator: ", ex);
            mToneGenerator = null;
        }
    }

    private void releaseTone() {
        if (mToneGenerator != null) {
            mToneGenerator.release();
            mToneGenerator = null;
        }
    }

    private void startTone() {
        ToneGenerator tg = mToneGenerator;
        if (tg != null) {
            tg.startTone(ToneGenerator.TONE_SUP_RINGTONE);
        }
    }

    private void stopTone() {
        ToneGenerator tg = mToneGenerator;
        if (tg != null) {
            tg.stopTone();
        }
    }

    /////////////////////////////////////////////////////
    //
    // Handset Monitor and Call Line Monitor
    //
    /////////////////////////////////////////////////////

    private void startMonitorCallLines() {
        mCallStatusListener = new MyCallStatusListener();
        BaseCallApi.addStatusListener("com.gs.apidemo.MonitorCall", mCallStatusListener.callback,
                PhoneContext.ListenType.LINE_ID | PhoneContext.ListenType.LINE_STATUS,true);
    }

    private void stopMonitorCallLines() {
        BaseCallApi.removeStatusListener(mCallStatusListener.callback);
        mCallStatusListener.destroy();
        mCallStatusListener = null;
    }

    private class MyCallStatusListener extends CallStatusListener {
        @Override
        public void onLineStatusChanged(int notifyType, BaseLine baseLine, List<BaseLine> list) {
            Log.i(TAG, "notifyType "+notifyType);
            Log.i(TAG, "lineInfo "+baseLine);

            final BaseLine line = baseLine;

            mNumberTv.setText(line.getRemoteNumber());
            mNameTv.setText(line.getRemoteName());
            mStatusTv.setText(String.valueOf(line.getStatus()));

            if (line.getStatus() == 2) { // ringing
                if (CallSettingApi.isAlwaysRingSpeaker()) {
                    AudioRouteApi.switchCurrentVoiceWithSpeaker();
                    // start ringing
                    startTone();
                    Log.i(TAG, "onLineStatusChanged: Start Ringing tone");
                }
            } else {
                if (mPrevStatus == 2) {
                    // stop ringing
                    stopTone();
                    Log.i(TAG, "onLineStatusChanged: Stop Ringing tone");
                    AudioRouteApi.switchCurrentVoiceWithoutSpeaker();
                }
            }

            mPrevStatus = line.getStatus();
            mPrevLine = line;
        }
    }

    private CallStatusListener mCallStatusListener = null;
}
